#!/bin/sh

. /usr/syno/sbin/wan_utils.sh

convert_isp_setting()
{
	old_wan_ifname="${1}"
	new_wan_ifname="${2}"
	old_class_id="/etc/dhcpc/dhcpcd-${old_wan_ifname}.classID"
	new_class_id="/etc/dhcpc/dhcpcd-${new_wan_ifname}.classID"
	old_client_id="/etc/dhcpc/dhcpcd-${old_wan_ifname}.clientID"
	new_client_id="/etc/dhcpc/dhcpcd-${new_wan_ifname}.clientID"
	old_options="/etc/dhcpc/dhcpcd-${old_wan_ifname}.option"
	new_options="/etc/dhcpc/dhcpcd-${new_wan_ifname}.option"
	old_hostname="/etc/dhcpc/dhcpcd-${old_wan_ifname}.host"
	new_hostname="/etc/dhcpc/dhcpcd-${new_wan_ifname}.host"
	old_6rd="/etc/dhcpc/dhcpcd-${old_wan_ifname}.6rd"
	new_6rd="/etc/dhcpc/dhcpcd-${new_wan_ifname}.6rd"

	cp "${old_class_id}" "${new_class_id}"
	cp "${old_client_id}" "${new_client_id}"
	cp "${old_options}" "${new_options}"
	cp "${old_hostname}" "${new_hostname}"
	cp "${old_6rd}" "${new_6rd}"

	mac=$(synogetkeyvalue /etc/synoinfo.conf "macclone_${old_wan_ifname}")
	if [ ! -z "${mac}" ]; then
		synosetkeyvalue /etc/synoinfo.conf "macclone_${new_wan_ifname}" "${mac}"
		ifconfig ${new_wan_ifname} down
		ifconfig ${new_wan_ifname} hw ether ${mac}
		ifconfig ${new_wan_ifname} up
	fi
}

wan_is_affected()
{
	old_wan0_is_affected=$(jq -r ".old.networks.wan0.is_affected" "${ARGS_FILE_PATH}")
	new_wan0_is_affected=$(jq -r ".new.networks.wan0.is_affected" "${ARGS_FILE_PATH}")
	old_wan1_is_affected=$(jq -r ".old.networks.wan1.is_affected" "${ARGS_FILE_PATH}")
	new_wan1_is_affected=$(jq -r ".new.networks.wan1.is_affected" "${ARGS_FILE_PATH}")
	old_br0_is_affected=$(jq -r ".old.networks.br0.is_affected" "${ARGS_FILE_PATH}")
	new_br0_is_affected=$(jq -r ".new.networks.br0.is_affected" "${ARGS_FILE_PATH}")

	if [ "${old_wan0_is_affected}" = "true" ] ||
	   [ "${old_wan1_is_affected}" = "true" ] ||
	   [ "${new_wan1_is_affected}" = "true" ] ||
	   [ "${new_br0_is_affected}" = "true" ] ||
	   [ "${old_br0_is_affected}" = "true" ] ||
	   [ "${new_wan0_is_affected}" = "true" ]; then
		return 0
	fi

	return 1
}

wan_topology_change_pre()
{
	old_context=$(jq -r ".old.context" "${ARGS_FILE_PATH}")

	if [ "${old_context}" = "repeater" ]; then
		return
	fi

	if ! wan_is_affected; then
		return
	fi

	stop_all_wan
}

wan_topology_change_post()
{
	new_context=$(jq -r ".new.context" "${ARGS_FILE_PATH}")

	if [ "${new_context}" = "repeater" ]; then
		return
	fi

	if ! wan_is_affected; then
		return
	fi

	old_context=$(jq -r ".old.context" "${ARGS_FILE_PATH}")
	old_wan0_ifname=$(jq -r ".old.networks.wan0.ifname" "${ARGS_FILE_PATH}")
	new_wan0_ifname=$(jq -r ".new.networks.wan0.ifname" "${ARGS_FILE_PATH}")
	new_wan0_link=$(jq -r ".new.networks.wan0.link" "${ARGS_FILE_PATH}")
	old_wan1_ifname=$(jq -r ".old.networks.wan1.ifname" "${ARGS_FILE_PATH}")
	new_wan1_ifname=$(jq -r ".new.networks.wan1.ifname" "${ARGS_FILE_PATH}")
	new_wan1_link=$(jq -r ".new.networks.wan1.link" "${ARGS_FILE_PATH}")
	has_link=""

	if [ "${old_wan0_ifname}" = "null" ] && [ "${old_context}" != "router" ]; then
		old_wan0_ifname="br0"
	fi
	if [ "${new_wan0_ifname}" = "null" ] && [ "${new_context}" != "router" ]; then
		new_wan0_ifname="br0"
		new_wan0_link=$(jq -r ".new.networks.br0.link" "${ARGS_FILE_PATH}")
	fi

	# assume only wan0 will change ifname
	if [ "${old_wan0_ifname}" != "null" ] && [ "${new_wan0_ifname}" != "null" ] &&
		[ "${old_wan0_ifname}" != "${new_wan0_ifname}" ]; then
		convert_isp_setting "${old_wan0_ifname}" "${new_wan0_ifname}"
		ip -4 addr flush dev "${old_wan0_ifname}"
		ip -6 addr flush dev "${old_wan0_ifname}" scope global
	fi

	# flush wan1 ip
	if [ "${old_wan1_ifname}" != "null" ] && [ "${new_wan1_ifname}" = "null" ]; then
		ip -4 addr flush dev "${old_wan1_ifname}"
		ip -6 addr flush dev "${old_wan1_ifname}" scope global
	fi

	if [ "${new_wan0_link}" = "true" ]; then
		has_link="$(synonettool --wan --ifname 0)"
	fi

	if [ "${new_wan1_link}" = "true" ]; then
		has_link="${has_link} $(synonettool --wan --ifname 1)"
	fi

	start_wan "${has_link}"
}

case $1 in
	--sdk-mod-ver)
	#Print SDK support version
	echo "1.0";
	;;
	--name)
	#Print package name
	echo "Wan topology change script"
	;;
	--pkg-ver)
	#Print package version
	echo "1.0";
	;;
	--vendor)
	#Print package vendor
	echo "Synology";
	;;
	--pre)
	wan_topology_change_pre
	;;
	--post)
	wan_topology_change_post
	;;
	*)
	echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
	;;
esac

